<?php

defined( 'oPage' ) or die();

/** class for image manipulation

    Supported Function
    - resize

    Supported Subsystems
    - ImageMagick (mogrify)
    - GD 1.x.x
    - GD 2.x.x

    Supported Image Types
    - ImageMagick: all
    - GD jpg, jpeg, png
    - GD gif (if GD supports it - not tested -  try GD 1.8.3)

    Notes:
    Using ImageMagick is my prefered methode (since GD has some limits).
    If you are running in SafeMode, system command will not work. So you have to use GD.
    I am using ImageMagick 5.4.6, since the other versions produce bigger images or have some errors.
    PNG Support in GD 1.x.x is limited to 256 colors.
*/
class CImage
{
    /** Grafic Subsystem used
        1: ImageMagick (default)
        2: GD
    */
    var $iSystem = 1;

    /** JPEG Quality
        0-100 (100 is best)
        85 (default)
    */
    var $iQuality = 85;

    /** Filename of image
    */
    var $sFilename;

    /** path to mogrify
        mogrify (default)
    */
    var $sIMMogrify = 'mogrify {options} {filename}';

    /** path to convert
    */
    var $sIMConvert = 'convert {options} {filename_old} {filename_new}';

    /** debug
        show debug messages
    */
    var $fDebug = false;

    /** resize only if size is different, default false
     */
    var $fResizeOnlyIfSizeIsDifferent = false;

    /** Constructor
        @param void
        @return void
    */
    function CImage( $sFilename = '' )
    {
        $this->SetFilename( $sFilename );
    }

    /** Set to subsystem ImageMagick
        @param void
        @return void
    */
    function UseImagemagick()
    {
        $this->iSystem = 1;
    }

    /** Set to subsystem GD
        @param void
        @return void
    */
    function UseGD()
    {
        $this->iSystem = 2;
    }

    function SetResizeOnlyIfSizeIsDifferent( $fValue = null )
    {
        if ( $fValue === null )
        {
            $fValue = true;
        }

        $this->fResizeOnlyIfSizeIsDifferent = $fValue;
    }

    /** Resizes the image using the given subsystem
        @param $iWidth width of new image, for no change use '' or 0
        @param $iHeight height of new image, for no change use '' or 0
        @param $fForce resize, even if image is already smaller
        @param $iMode default 0: default mode, 1: $iWidth and $iHeight are minimum values, 2: $iWidth and $iHeight are maximum values
        @return true/false
    */
    function Resize( $iWidth, $iHeight = 0, $fForce = null, $iMode = null )
    {
        // check parameters
        if ( $iWidth == '' || $iWidth < 0 )
        {
            $iWidth = 0;
        }
        if ( $iHeight == '' || $iHeight < 0 )
        {
            $iHeight = 0;
        }
        if ( $fForce === null )
        {
            $fForce = false;
        }
        if ( $iMode === null )
        {
            $iMode = 0;
        }

        // Minimum mode
        switch( $iMode )
        {
            case 1:
                $iMinWidth = $iWidth;
                $iMinHeight = $iHeight;
                $iMaxWidth = 0;
                $iMaxHeight = 0;
                $iHeight = 0;
            break;

            case 2:
                $iMinWidth = 0;
                $iMinHeight = 0;
                $iMaxWidth = $iWidth;
                $iMaxHeight = $iHeight;
                $iHeight = 0;
            break;

            default:
                $iMinWidth = 0;
                $iMinHeight = 0;
                $iMaxWidth = 0;
                $iMaxHeight = 0;
            break;
        }

        // get filename
        $sFilename = $this->GetFilename();

        // calculate actual width and height of the new image
        $aSize = GetImageSize( $sFilename );
        if ( $iWidth == 0 && $iHeight == 0 )
        {
            // no resize wanted
            $iWidth = $aSize[ 0 ];
            $iHeight = $aSize[ 1 ];
        }

        if ( !$fForce && $iWidth > $aSize[ 0 ] )
        {
            $iWidth = $aSize[ 0 ];
        }
        if ( !$fForce && $iHeight > $aSize[ 1 ] )
        {
            $iHeight = $aSize[ 1 ];
        }

        if ( $iWidth == 0 && $iHeight != 0 )
        {
            // calculate right width
            $iWidth = $iHeight / $aSize[ 1 ] * $aSize[ 0 ];
        }
        if ( $iWidth != 0 && $iHeight == 0 )
        {
            // calculate right height
            $iHeight = $iWidth / $aSize[ 0 ] * $aSize[ 1 ];
        }

        // check, if current image is smaller then new image
        if ( $fForce )
        {
            if ( $iWidth > $aSize[ 0 ] && $iHeight > $aSize[ 1 ] )
            {
                $iWidth = $aSize[ 0 ];
                $iHeight = $aSize[ 1 ];
            }
        }

        // Check minimum values
        if ( $iMinWidth > 0 && $iWidth < $iMinWidth )
        {
            $iWidth = $iMinWidth;
            $iHeight = $iWidth / $aSize[ 0 ] * $aSize[ 1 ];
        }
        if ( $iMinHeight > 0 && $iHeight < $iMinHeight )
        {
            $iHeight = $iMinHeight;
            $iWidth = $iHeight / $aSize[ 1 ] * $aSize[ 0 ];
        }

        // Check maximum values
        if ( $iMaxWidth > 0 && $iWidth > $iMaxWidth )
        {
            $iWidth = $iMaxWidth;
            $iHeight = $iWidth / $aSize[ 0 ] * $aSize[ 1 ];
        }
        if ( $iMaxHeight > 0 && $iHeight > $iMaxHeight )
        {
            $iHeight = $iMaxHeight;
            $iWidth = $iHeight / $aSize[ 1 ] * $aSize[ 0 ];
        }

        $iWidth = round( $iWidth );
        $iHeight = round( $iHeight );

        // Do resize only, if width and height are different to actual size
        if ( $this->fResizeOnlyIfSizeIsDifferent == false || ( $this->fResizeOnlyIfSizeIsDifferent == true && ( $iWidth != $aSize[ 0 ] || $iHeight != $aSize[ 1 ] ) ) )
        {
            switch( $this->iSystem )
            {
                // ImageMagick
                case 1:
                    $fReturn = $this->IMResize( $sFilename, $iWidth, $iHeight, $this->GetQuality(), $fForce );
                break;

                // GD
                case 2:
                    $fReturn = $this->GDResize( $sFilename, $iWidth, $iHeight, $this->GetQuality(), $fForce );
                break;

                default:
                    return( false );
                break;
            }
        }
        else
        {
            $fReturn = true;
        }

        return( $fReturn );
    }

    /** Resizes an imagefile
        Supported types: all ImageMagick supports
        Tested with ImageMagick Versions 5.4.6
        @param $sFilename Filename of an image
        @param $iWidth Width of new image
        @param $iHeight Height of new image
        @iQuality JPG quality (default 85)
        @return true/false
    */
    function IMResize( $sFilename, $iWidth, $iHeight, $iQuality = Null, $fForce = Null )
    {
        // if the filename is invalid, just exit
        if ( !file_exists( $sFilename ) )
            return( false );

        if ( $iQuality === null )
            $iQuality = 85;
        if ( $fForce === null )
            $fForce = false;

        // mogrify.exe -colorspace RGB +profile \* -density 72x72 -geometry 100x100! -quality 75 image.jpg
        // Options are processed in the following order:
        // 1. -colorspace RGB: convert image into RGB (Internet Explorer can not display CMYK)
        // 2. +profile "*": quoting the * is needed, since on unix a single * is matched against all files in the current directory
        // 3. -density 72x72: say, that the image is 72 DPI
        // 4. -geometry 100x100!: make the image a given size
        // 5. -quality 75: image quality
        $sOptions = '-colorspace RGB +profile "*" -density 72x72 -geometry ' . $iWidth . 'x' . $iHeight . '! -quality ' . $iQuality;

        $sCommand = $this->GetIMMogrify();
        $sCommand = str_replace( '{options}', $sOptions, $sCommand );
        $sCommand = str_replace( '{filename}', '"' . $sFilename . '"', $sCommand );

        $iReturn = 0;
        // echo $sCommand . "<br>\n";
        system( $sCommand, $iReturn );
        // echo 'Return: ' . $iReturn . "<br>\n";

        if ( ini_get( 'safe_mode' ) && $iReturn != 0 )
            trigger_error( 'IMResize: Resize failed, safe mode is active -> check safe_mode_exec_dir in php.ini or httpd.conf!', E_USER_NOTICE );

        /*
        $sBuffer = $this->System( $sCommand );
        echo $sCommand . "<br>\n";
        echo $sBuffer . "<br>\n";
        */

        if ( $iReturn != 0 )
            return( false );

        chmod( realpath( $sFilename ), 0666 );

        return( true );
    }

    /*
    function System( $command )
    {
        if ( $oProc = popen( "($command) 2>&1", "r" ) )
        {
            $sResult = '';
            while( !feof( $oProc ) )
                $sResult .= fgets( $oProc, 1000 );
            pclose( $oProc );

            return( $sResult );
        }
    }
    */

    /** Resizes an imagefile
        Supported types: jpg, jpeg, png
        gif is only supported, if GD does support it.
        Tested with GD Versions 1.6.2 and 2.0.1
        @param $sFilename Filename of an image
        @param $iWidth Width of new image
        @param $iHeight Height of new image
        @iQuality JPG quality (default 85)
        @return true/false
    */
    function GDResize( $sFilename, $iWidth, $iHeight, $iQuality = 85, $fForst = false )
    {
        // if the filename is invalid, just exit
        if ( !file_exists( $sFilename ) )
            return( false );

        // load image
        $sExtension = Extension( $sFilename );
        switch( $sExtension )
        {
            case 'gif':
                // only resize gif, if system supports it.
                if ( !function_exists( 'imagecreatefromgif' ) )
                    return( false );
                $oImage = ImageCreateFromGif( $sFilename );
            break;

            case 'jpg':
            case 'jpeg':
                $oImage = ImageCreateFromJPEG( $sFilename );
            break;

            case 'png':
                $oImage = ImageCreateFromPNG( $sFilename );
            break;

            default:
                return( false );
            break;
        }

        // Resize is time consuming: Only do it, when it's necessary
        $aSize = GetImageSize( $sFilename );
        if ( $aSize[ 0 ] > $iWidth || $aSize[ 1 ] > $iHeight || $fForce )
            if ( !$this->GDResizeCubic( $oImage, $iWidth, $iHeight ) )
            {
                ImageDestroy( $oImage );
                return( false );
            }

        switch( $sExtension )
        {
            case 'gif':
                if ( function_exists( 'imagegif' ) )
                    ImageGIF( $oImage, $sFilename, 85 );
            break;

            case 'jpg':
            case 'jpeg':
                ImageJPEG( $oImage, $sFilename, $iQuality );
            break;

            case 'png':
                ImagePNG( $oImage, $sFilename );
            break;

            default:
            break;
        }

        ImageDestroy( $oImage );

        return( true );
    }

    /** Resizes a given image with bicubic resample
        @param &$oImage reference to a GD image, gets replaced by new image
        @param $iWidth width of new image
        @param $iHeight heigth of new image
        @return true/false
    */
    function GDResizeCubic( &$oImage, $iWidth, $iHeight )
    {
        $oImageNew = @ImageCreateTruecolor( $iWidth, $iHeight );
        if ( !$oImageNew)
            $oImageNew = ImageCreate( $iWidth, $iHeight );

        // Copy colors from original image
        ImagePaletteCopy( $oImageNew, $oImage );

        // Calculate scale factors
        $iScaleX = ( imagesx( $oImage ) -1 ) / $iWidth;
        $iScaleY = ( imagesy( $oImage ) -1 ) / $iHeight;
        $iScaleX2 = intval( $iScaleX / 2 );
        $iScaleY2 = intval( $iScaleY / 2 );

        // Calculate new color for each pixel of new image
        for( $j = 0; $j < $iHeight; $j++ )
        {
            $iY = intval( $j * $iScaleY );
            $y13 = $iY + $iScaleY2;

            for( $i = 0; $i < $iWidth; $i++ )
            {
                $iX = intval( $i * $iScaleX );
                $x34 = $iX + $iScaleX2;

                $c1 = ImageColorsForIndex( $oImage, ImageColorAt( $oImage, $iX, $y13 ) );
                $c2 = ImageColorsForIndex( $oImage, ImageColorAt( $oImage, $iX, $iY ) );
                $c3 = ImageColorsForIndex( $oImage, ImageColorAt( $oImage, $x34, $y13 ) );
                $c4 = ImageColorsForIndex( $oImage, ImageColorAt( $oImage, $x34, $iY ) );

                $r = intval( ( $c1[ 'red' ] + $c2[ 'red' ] + $c3[ 'red' ] + $c4[ 'red' ] ) / 4 );
                $g = intval( ( $c1[ 'green' ] + $c2[ 'green' ] + $c3[ 'green' ] + $c4[ 'green' ] ) / 4 );
                $b = intval( ( $c1[ 'blue' ] + $c2[ 'blue' ] + $c3[ 'blue' ] + $c4[ 'blue' ] ) / 4 );

                ImageSetPixel( $oImageNew, $i, $j, ImageColorClosest( $oImageNew, $r, $g, $b ) );
            }
        }

        // Free resource (important!)
        ImageDestroy( $oImage );

        // Copy image reference
        $oImage = $oImageNew;

        return( true );
    }

    /** Converts the image using the given subsystem
        @param $sFilename Target file (extension defines desired format)
        @return true/false
    */
    function Convert( $sFilename )
    {
        switch( $this->iSystem )
        {
            // ImageMagick
            case 1:
                $fReturn = $this->IMConvert( $this->GetFilename(), $sFilename, $this->GetQuality() );
            break;

            // GD
            case 2:
                return( false );
            break;

            default:
                return( false );
            break;
        }

        return( $fReturn );
    }

    /** Converts an imagefile
        Supported types: all ImageMagick supports
        Tested with ImageMagick Versions 5.4.6
        @param $sFrom Filename of an image
        @param $sTo Filename of the target image
        @iQuality JPG quality (default 85)
        @return true/false
    */
    function IMConvert( $sFrom, $sTo, $iQuality = null )
    {
        // if the filename is invalid, just exit
        if ( !file_exists( $sFrom ) )
            return( false );

        if ( $iQuality === null )
            $iQuality = 85;

        $sOptions = '-colorspace RGB -quality ' . $iQuality;

        $sCommand = $this->GetIMConvert();
        $sCommand = str_replace( '{options}', $sOptions, $sCommand );
        $sCommand = str_replace( '{filename_old}', '"' . $sFrom . '"', $sCommand );
        $sCommand = str_replace( '{filename_new}', '"' . $sTo . '"', $sCommand );

        system( $sCommand, $iReturn );

        // $sBuffer = $this->System( $sCommand );
        // echo $sCommand . "<br>\n";
        // echo $sBuffer . "<br>\n";
        // echo "Return: $iReturn<br>";

        if ( $iReturn != 0 )
            return( false );

        chmod( realpath( $sTo ), 0666 );

        return( true );
    }

    /** Set/Get for $iQuality
    */
    function SetQuality( $iValue )
    {
        if ( $iValue < 0 )
            $iValue = 0;
        if ( $iValue > 100 )
            $iValue = 100;
        $this->iQuality = $iValue;
    }
    function GetQuality()
    {
        return( $this->iQuality );
    }

    /** Set/Get for $sFilename
    */
    function SetFilename( $sValue )
    {
        $this->sFilename = $sValue;
    }
    function GetFilename()
    {
        return( $this->sFilename );
    }

    /** Set/Get for $sIMConvert
    */
    function SetIMConvert( $sValue )
    {
        $this->sIMConvert = $sValue;
    }
    function GetIMConvert()
    {
        return( $this->sIMConvert );
    }

    /** Set/Get for $sIMMogrify
    */
    function SetIMMogrify( $sValue )
    {
        $this->sIMMogrify = $sValue;
    }
    function GetIMMogrify()
    {
        return( $this->sIMMogrify );
    }

    function SetDebug( $fValue = null )
    {
        if ( $fValue === null )
            $fValue = true;

        $this->fDebug = $fValue;
    }
    function GetDebug()
    {
        return( $this->fDebug );
    }
}
